drop table if exists #MLB2019RegularSeason;
create table #MLB2019RegularSeason
(
	ID int not null identity(1,1),
	TeamName nvarchar(40) not null,
	League nvarchar(10) not null,
	Division nvarchar(10) not null,
	Wins int not null,
	Losses int not null
);

insert #MLB2019RegularSeason (TeamName, League, Division, Wins, Losses)
values
('New York Yankees','American','East',103,59),
('Tampa Bay Rays','American','East',96,66),
('Boston Red Sox','American','East',84,78),
('Toronto Blue Jays','American','East',67,95),
('Baltimore Orioles','American','East',54,108),
('Minnesota Twins','American','Central',101,61),
('Cleveland Indians','American','Central',93,69),
('Chicago White Sox','American','Central',72,89),
('Kansas City Royals','American','Central',59,103),
('Detroit Tigers','American','Central',47,114),
('Houston Astros','American','West',107,55),
('Oakland Athletics','American','West',97,65),
('Texas Rangers','American','West',78,84),
('Los Angeles Angels','American','West',72,90),
('Seattle Mariners','American','West',68,94),
('Atlanta Braves','National','East',97,65),
('Washington Nationals','National','East',93,69),
('New York Mets','National','East',86,76),
('Philadelphia Phillies','National','East',81,81),
('Miami Marlins','National','East',57,105),
('St. Louis Cardinals','National','Central',91,71),
('Milwaukee Brewers','National','Central',89,73),
('Chicago Cubs','National','Central',84,78),
('Cincinnati Reds','National','Central',75,87),
('Pittsburgh Pirates','National','Central',69,93),
('Los Angeles Dodgers','National','West',106,56),
('Arizona Diamondbacks','National','West',85,77),
('San Francisco Giants','National','West',77,85),
('Colorado Rockies','National','West',71,91),
('San Diego Padres','National','West',70,92);

-- Compare ROW_NUMBER with RANK and DENSE_RANK.
-- Ranking is being done by the Wins column.
-- Note that overall sort order is by ID.
select TeamName, Wins, Losses,	
	row_number() over (order by Wins desc) TeamRowNumber,
	rank() over (order by Wins desc) TeamRank,
	dense_rank() over (order by Wins desc) TeamDenseRank
from #MLB2019RegularSeason
order by ID;

-- Easier to visualize if we order by the ROW_NUMBER value.
-- Note differences in ROW_NUMBER vs RANK.
-- Note differences in RANK vs DENSE_RANK.
select TeamName, Wins, Losses,	
	row_number() over (order by Wins desc) TeamRowNumber,
	rank() over (order by Wins desc) TeamRank,
	dense_rank() over (order by Wins desc) TeamDenseRank
from #MLB2019RegularSeason
order by row_number() over (order by Wins desc);

-- Add NTILE in the query.
select TeamName, Wins, Losses,	
	row_number() over (order by Wins desc) TeamRowNumber,
	rank() over (order by Wins desc) TeamRank,
	dense_rank() over (order by Wins desc) TeamDenseRank,
	ntile(5)over (order by Wins desc) TeamNTile
from #MLB2019RegularSeason
order by row_number() over (order by Wins desc);

-- ROW_NUMBER requires the ORDER BY clause (PARTITION BY is optional).
-- If we don't care about order, still need an ORDER BY. Use a constanct.
select TeamName, Wins, Losses,	
	row_number() over (order by (select 0)) TeamRowNumber
from #MLB2019RegularSeason
order by ID;

-- Both PARTITION BY and ORDER BY support multiple columns
select League, Division, TeamName, Wins, Losses,	
	row_number()
		over (
			partition by League, Division
			order by Wins desc) TeamRowNumber
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc);
