drop table if exists #sales;
create table #sales
(
	ID int not null identity(1,1),
	CustId int not null,
	OrderDate date not null,
	Price numeric(7,2) not null,
	RTotal numeric(7,2) null
);

insert #sales (CustID, OrderDate, Price)
values
(1, '2019-01-10', 1.00),
(1, '2019-02-11', 2.10),
(1, '2019-03-12', 3.20),
(1, '2019-04-13', 4.30),
(1, '2019-05-14', 5.40),
(2, '2019-01-20', 6.50),
(2, '2019-01-21', 7.60);

-- Old-fashioned aggregate using GROUP BY.
select CustId,
	sum(Price) Total
from #sales
group by CustId;

-- Even without GROUP BY, it can still be old-style aggregate.
select sum(Price) Total
from #sales;

-- The OVER clause is what makes it a window function.
select CustID,
	OrderDate,
	Price,
	sum(Price)
		over (partition by CustId
		order by OrderDate) as RTotal
from #sales;

-- Try it without the PARTITION BY.
select CustID,
	OrderDate,
	Price,
	sum(Price)
		over (order by OrderDate) as RTotal
from #sales;

-- Try it without the ORDER BY.
select CustID,
	OrderDate,
	Price,
	sum(Price)
		over (partition by CustId) as RTotal
from #sales;

-- Can even omit both both of them
select CustID,
	OrderDate,
	Price,
	sum(Price)
		over () as RTotal
from #sales;

-- Add an arbitrary frame
select CustID,
	OrderDate,
	Price,
	sum(Price)
		over (partition by CustId
		order by OrderDate
		rows between 1 preceding and current row) as RTotal
from #sales;
