drop table if exists #MLB2019RegularSeason;
create table #MLB2019RegularSeason
(
	ID int not null identity(1,1),
	TeamName nvarchar(40) not null,
	League nvarchar(10) not null,
	Division nvarchar(10) not null,
	Wins int not null,
	Losses int not null
);

insert #MLB2019RegularSeason (TeamName, League, Division, Wins, Losses)
values
('New York Yankees','American','East',103,59),
('Tampa Bay Rays','American','East',96,66),
('Boston Red Sox','American','East',84,78),
('Toronto Blue Jays','American','East',67,95),
('Baltimore Orioles','American','East',54,108),
('Minnesota Twins','American','Central',101,61),
('Cleveland Indians','American','Central',93,69),
('Chicago White Sox','American','Central',72,89),
('Kansas City Royals','American','Central',59,103),
('Detroit Tigers','American','Central',47,114),
('Houston Astros','American','West',107,55),
('Oakland Athletics','American','West',97,65),
('Texas Rangers','American','West',78,84),
('Los Angeles Angels','American','West',72,90),
('Seattle Mariners','American','West',68,94),
('Atlanta Braves','National','East',97,65),
('Washington Nationals','National','East',93,69),
('New York Mets','National','East',86,76),
('Philadelphia Phillies','National','East',81,81),
('Miami Marlins','National','East',57,105),
('St. Louis Cardinals','National','Central',91,71),
('Milwaukee Brewers','National','Central',89,73),
('Chicago Cubs','National','Central',84,78),
('Cincinnati Reds','National','Central',75,87),
('Pittsburgh Pirates','National','Central',69,93),
('Los Angeles Dodgers','National','West',106,56),
('Arizona Diamondbacks','National','West',85,77),
('San Francisco Giants','National','West',77,85),
('Colorado Rockies','National','West',71,91),
('San Diego Padres','National','West',70,92);

-- Compare each team to the next best team in the same division.
-- The LAG function returns the column value from the previous row.
select League, Division, TeamName, Wins,
	lag(Wins)
		over (
			partition by League, Division
			order by Wins desc) NextBestTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- If for some reason we want to look back TWO rows, can do that as well.
select League, Division, TeamName, Wins,
	lag(Wins, 2)
		over (
			partition by League, Division
			order by Wins desc) SecondToNextBestTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- We can also specifiy a default value.
select League, Division, TeamName, Wins,
	lag(Wins, 1, 0)
		over (
			partition by League, Division
			order by Wins desc) NextBestTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- Instead of a constant, use a column value
select League, Division, TeamName, Wins,
	lag(Wins, 1, Wins)
		over (
			partition by League, Division
			order by Wins desc) NextBestTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- The LEAD function returns the column value from the next row.
select League, Division, TeamName, Wins,
	lead(Wins)
		over (
			partition by League, Division
			order by Wins desc) NextWorstTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- The FIRST_VALUE function returns the first value for the column
-- within the current frame.
select League, Division, TeamName, Wins,
	first_value(Wins)
		over (
			partition by League, Division
			order by Wins desc) TopTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- Use FIRST_VALUE to calculate games behind.
with Stats as
(
select ID, League, Division, TeamName, Wins, Losses,
	first_value(Wins)
		over (
			partition by League, Division
			order by Wins desc) TopTeamWins,
	first_value(Losses)
		over (
			partition by League, Division
			order by Wins desc) TopTeamLosses
from #MLB2019RegularSeason
)
select League, Division, TeamName, Wins, Losses,
	cast(((TopTeamWins - TopTeamLosses) - (Wins - Losses)) / 2.0 as numeric(4,1))
from Stats
order by League, Division, row_number() over (order by Wins desc, ID);

-- The LAST_VALUE function returns the last value for the column
-- within the current frame.
select League, Division, TeamName, Wins,
	last_value(Wins)
		over (
			partition by League, Division
			order by Wins desc) BottomTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);




















-- Wait, what's going on here?
select League, Division, TeamName, Wins,
	last_value(Wins)
		over (
			partition by League, Division
			order by Wins desc
			rows between unbounded preceding and unbounded following) BottomTeamWins
from #MLB2019RegularSeason
order by League, Division, row_number() over (order by Wins desc, ID);

-- Get post-season teams
-- Ignore ties
with DivisionLeaders as
(
	select *
	from
	(
		select *,
			row_number() over (partition by League, Division order by Wins desc) rn
		from #MLB2019RegularSeason
	) mlb
	where rn = 1
), Wildcards as
(
	select *
	from
	(
		select *,
			row_number() over (partition by League order by Wins desc) rn
		from #MLB2019RegularSeason
		where ID not in (select ID from DivisionLeaders)
	) mlb
	where rn <= 2
)
select dl.League, dl.Division, dl.TeamName, dl.Wins, dl.Losses, 'Division champ' as Reason
from DivisionLeaders dl
union all
select w.League, w.Division, w.TeamName, w.Wins, w.Losses, 'Wildcard' as Reason
from Wildcards w
order by League, Wins desc;

